package com.telpo.dipperposition.handler;

import com.telpo.dipperposition.config.PositionConfig;
import com.telpo.dipperposition.service.IDipperAstPosAsyncTaskService;
import com.telpo.dipperposition.service.IDipperAstTimeAsyncTaskService;
import com.telpo.dipperposition.service.IDipperDataAsyncTaskService;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.CompositeByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * @program: dipperposition
 * @description: Netty服务器处理句柄
 * @author: linwl
 * @create: 2021-01-13 13:56
 **/
@Slf4j
@Component
public class NettyServerHandler extends ChannelInboundHandlerAdapter {

    private static String AST_TIME_CMD = "TIME";
    private static String AST_POS_CMD = "POS";
    private static String AST_EPH_CMD = "EPH";
    private static String AST_ALL_CMD = "all";
    private static String AST_TIME_CMD_BYTE = "54494d45";
    private static String AST_POS_CMD_BYTE = "504f53";
    private static String AST_EPH_CMD_BYTE = "455048";
    private static String AST_ALL_CMD_BYTE = "616C6C";

    @Autowired
    private PositionConfig positionConfig;
    @Autowired
    private IDipperAstTimeAsyncTaskService dipperTimeAsyncTaskService;
    @Autowired
    private IDipperAstPosAsyncTaskService dipperAstPosAsyncTaskService;
    @Autowired
    private IDipperDataAsyncTaskService dipperDataAsyncTaskService;

    private static NettyServerHandler nettyServerHandler;
    public NettyServerHandler() {}
    @PostConstruct
    public void init() {
        nettyServerHandler = this;
        nettyServerHandler.positionConfig = this.positionConfig;
        nettyServerHandler.dipperTimeAsyncTaskService = this.dipperTimeAsyncTaskService;
        nettyServerHandler.dipperAstPosAsyncTaskService = this.dipperAstPosAsyncTaskService;
        nettyServerHandler.dipperDataAsyncTaskService = this.dipperDataAsyncTaskService;
    }

    /**
     * 客户端连接会触发
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Channel active......");

        SocketChannel channel = (SocketChannel) ctx.channel();
        log.info("链接报告开始");
        log.info("链接报告信息：有一客户端链接到本服务端");
        log.info("链接报告IP:" + channel.localAddress().getHostString());
        log.info("链接报告Port:" + channel.localAddress().getPort());
        log.info("链接报告完毕");
        //通知客户端链接建立成功
        // 默认返回取得时间成功
        String ackAckCheckRef = "233E0101020004020A1D";

//        if (Integer.parseInt(nettyServerConfig.getPosAsycServerPort()) == channel.localAddress().getPort()) {
//            ackAckCheckRef = "233E010102000401091C";
//        }
//        if (Integer.parseInt(nettyServerConfig.getStarsAsycServerPort()) == channel.localAddress().getPort()) {
//            ackAckCheckRef = "233E010102000421293C";
//        }

        if (9013 == channel.localAddress().getPort()) {
            ackAckCheckRef = "233E010102000401091C";
        }
        if (9012 == channel.localAddress().getPort()) {
            ackAckCheckRef = "233E010102000421293C";
        }
        //String str = "通知客户端链接建立成功" + " " + LocalDateTime.now() + " " + channel.localAddress().getHostString() +
        // "\r\n";
        ByteBuf buf = Unpooled.buffer(ackAckCheckRef.getBytes().length);
        buf.writeBytes(ackAckCheckRef.getBytes(CharsetUtil.UTF_8));
        ctx.writeAndFlush(buf);
    }


    /**
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.info("客户端断开链接，IP：{}", ctx.channel().localAddress().toString());
    }

    /**
     * 客户端发消息会触发
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        //接收msg消息与上一章节相比，此处已经不需要自己进行解码}
        SocketChannel channel = (SocketChannel) ctx.channel();
        String ipAddress = channel.remoteAddress().getHostString();
        log.info("接收到消息:" + msg + ",客户端IP:" + ipAddress);
        if(ipAddress.contains(":")) {
            String[] ipAddressArr = ipAddress.split(":");
            ipAddress = ipAddressArr[0].replace("/","");
        }
        log.info("新客户端IP:" + ipAddress);

        String channelAns = "";
        // 返回时间指令
        ByteBuf buf = null;
        if (AST_TIME_CMD.equals(msg)) {
            // 初始时间辅助输入;
            channelAns = nettyServerHandler.dipperTimeAsyncTaskService.pushAstTime();
            //log.debug(channelAns);
            if (channelAns != null) {
                buf = Unpooled.buffer(channelAns.getBytes().length);
                buf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                ctx.writeAndFlush(buf);
            }
        }
        // 发送SDBP-AST-POS获取辅助位置信息
        if (AST_POS_CMD.equals(msg)) {
            channelAns = nettyServerHandler.dipperAstPosAsyncTaskService.pushAstPos(ipAddress);
            //log.debug(channelAns);
            if (channelAns != null) {
                buf = Unpooled.buffer(channelAns.getBytes().length);
                buf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                ctx.writeAndFlush(buf);
            }
        }

        // 从缓存获取SDBP-AST-EPH星历数
        if (AST_EPH_CMD.equals(msg)) {
            channelAns = nettyServerHandler.dipperDataAsyncTaskService.getAstEPH();
            //log.debug(channelAns);
            if (channelAns != null) {
                buf = Unpooled.buffer(channelAns.getBytes().length);
                buf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                ctx.writeAndFlush(buf);
            }
        }

        // 最后把SDBP-AST-TIME、SDBP-AST-POS、SDBP-AST-EPH并包一起发给设备。
        // 设备采用16进制获取数据，则代理服务器也是采用16进制返回数据。
        // 通知客户端链消息发送成功
        if (AST_ALL_CMD.equals(msg)) {
            CompositeByteBuf  compositeByteBuf = Unpooled.compositeBuffer();
            channelAns = nettyServerHandler.dipperTimeAsyncTaskService.pushAstTime();

            if (channelAns != null) {
                ByteBuf channelTimeAnsBuf = Unpooled.buffer(channelAns.getBytes().length);
                channelTimeAnsBuf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                compositeByteBuf.addComponent(channelTimeAnsBuf);
            }
            channelAns = nettyServerHandler.dipperAstPosAsyncTaskService.pushAstPos(ipAddress);

            if (channelAns != null) {
                ByteBuf channelPosAnsBuf = Unpooled.buffer(channelAns.getBytes().length);
                channelPosAnsBuf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                compositeByteBuf.addComponent(channelPosAnsBuf);
            }
            channelAns = nettyServerHandler.dipperDataAsyncTaskService.getAstEPH();
            if (channelAns != null) {
                ByteBuf channelPehAnsBuf = Unpooled.buffer(channelAns.getBytes().length);
                channelPehAnsBuf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                compositeByteBuf.addComponent(channelPehAnsBuf);
            }
            ctx.writeAndFlush(compositeByteBuf);
        }

    }

    /**
     * 发生异常触发
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }


}
